function generate_cmi_mi_pv_function()
disp('Loading null data ... ');
load('cmi_mi_null_dist.mat');

% absolute delta MIs
abs_mu    = mean(abs(null_dist));
abs_sigma = std(abs(null_dist));

% figure;
% plot(bin_stat(:,3),abs_mu,'b',bin_stat(:,3),abs_sigma,'r','LineWidth',2);
% xlabel('\it{I}','FontSize',20);
% ylabel('|\Delta\it{I}| Statistics','FontSize',20);
% legend('Mean','Standard Deviation','location','SouthEast');
% set(gca,'FontSize',20);
% pause(0.1);
% awtinvoke(get(gcf,'javaframe'),'maximize');
% print -painters -dmeta -r300 figure_NKI_absolute_detla_mi_statistics.wmf

% original delta MIs
mu    = mean(null_dist);
sigma = std(null_dist);

% figure;
% plot(bin_stat(:,3),mu,'b',bin_stat(:,3),sigma,'r','LineWidth',2);
% xlabel('\it{I}','FontSize',20);
% ylabel('\Delta\it{I} Statistics','FontSize',20);
% legend('Mean','Standard Deviation','location','NorthWest');
% set(gca,'FontSize',20);
% pause(0.1);
% awtinvoke(get(gcf,'javaframe'),'maximize');
% print -painters -dmeta -r300 figure_NKI_detla_mi_statistics.wmf

% fitting CDF to each bin
ftype = fittype('-a*x.^n+b');
opts = fitoptions(ftype);
opts.StartPoint = [64 0 1.5];   % initial values

a    = zeros(nbins,1);
b    = zeros(nbins,1);
n    = zeros(nbins,1);
data = abs(null_dist);
N    = size(data,1);   % total number of points in each bin
disp('Fitting bin CDFs ... ');
buffer = ' ';
figure;
for i=1:nbins
    str = num2str(i);
    disp([repmat(char(8),1,length(buffer)) str]);
    buffer = [str ' '];
    
    [c,I] = sort(data(:,i));
    idx   = 100:100:N;
    cumf  = zeros(length(idx),2);
    w     = zeros(length(idx),1);  
    for j=1:length(idx)
        cumf(j,1) = c(idx(j));
        cumf(j,2) = (N-idx(j))/N;    
        if j==1
            w(j) = c(idx(j));
        else
            w(j) = c(idx(j))-c(idx(j-1));
        end
    end
    ids = find(cumf(:,2)~=0);
    xx  = cumf(ids,1);
    yy  = log10(cumf(ids,2));
    opts.Weights = w(ids)./sum(w(ids));
    fresult = fit(xx,yy,ftype,opts);
    % f(x) = -a*x^n+b;
    a(i) = fresult.a;
    b(i) = fresult.b;
    n(i) = fresult.n;
    
%     if (mod(i-1,10) == 0)
%         subplot(5,2,(i-1)/10+1);
%         semilogy(cumf(:,1),cumf(:,2),'b.','MarkerSize',12);
%         title(['Bin #' num2str(i) ', \it{\bf{I}}_{total}~' num2str(bin_stat(i,3))],'FontSize',16);
%         hold on;
%         y_hat = (-fresult.a)*(xx.^fresult.n) + fresult.b;
%         semilogy(xx,power(10,y_hat),'r');
%         hold off;
%         if ((i-1)/10+1 >= 9)
%             xlabel('|\Delta\it{\bf{I}}|','FontSize',16);
%         end
%         set(gca,'ylim',[0.0001,1],'FontSize',16);        
%     end
end
% pause(0.1);
% awtinvoke(get(gcf,'javaframe'),'maximize');
% print -painters -dmeta -r300 figure_NKI_delta_mi_bin_fitting.wmf

t = bin_stat(:,2);  % upper bound of each bin

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Generate a MATLAB function that compute p-value of delta MIs
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% log10[p-value] = -a*[mi]^n+b
% p-value = 10^(-a*[mi]^n+b);
outfile = 'pvalue.m';
disp('Generating p-value function ...');
out = fopen(outfile ,'w');

fprintf(out,'%s\n\n', 'function p = pvalue(mi,cmi);');

fprintf(out,'%s\n', '% P-value function for Test dataset dataset');
%fprintf(out,'%s\n\n', ['% Tail size: ' num2str(q)]);

fprintf(out,'%s\n','if isempty(mi)');
fprintf(out,'\t%s\n','p = [];');
fprintf(out,'\t%s\n','return;');
fprintf(out,'%s\n\n','end');

fprintf(out,'%s\n','if (~isvector(mi) | size(mi,1)<size(mi,2))');
fprintf(out,'\t%s\n','error(''MI must be a column vector!'');');
fprintf(out,'%s\n\n','end');
fprintf(out,'%s\n','if length(mi)~=size(cmi,1)');
fprintf(out,'\t%s\n','error(''Conditional MI matrix must have same number of rows as the MI vector!'');');
fprintf(out,'%s\n\n','end');

fprintf(out,'%s\n\n',['a = ' mat2str(a) ';']);
fprintf(out,'%s\n\n',['b = ' mat2str(b) ';']);
fprintf(out,'%s\n\n',['n = ' mat2str(n) ';']);
fprintf(out,'%s\n\n',['t = ' mat2str(t) ';']);

fprintf(out,'%s\n','c = zeros(length(mi),1);');
fprintf(out,'%s\n','for i=1:length(c)');
fprintf(out,'\t%s\n','j = 1;');
fprintf(out,'\t%s\n',['while (j<' num2str(nbins) ' & mi(i)>=t(j))']);
fprintf(out,'\t\t%s\n','j = j+1;');
fprintf(out,'\t%s\n','end');
fprintf(out,'\t%s\n','c(i) = j;');
fprintf(out,'%s\n','end');
fprintf(out,'%s\n','idx = c*ones(1,size(cmi,2));');
fprintf(out,'%s\n',['p = 10.^(b(idx)-a(idx).*(abs(cmi).^n(idx)));']);
fclose(out);
