function generate_delta_mi_pv_null_dist(data,tail,method_mutualinfo)

rand('state',sum(100*clock));

[ng,ma] = size(data);

q  = tail;
dn = 1:floor(q*ma);
up = (ma-floor(q*ma)+1):ma;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Reading parameters of config_kernel file
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
fid = fopen('config_kernel.txt');
contents = textscan(fid,'%s','delimiter','\n');
fclose(fid);
contents = contents{1};
cline = strmatch('>',contents);
contents(cline) = [];
parameters_kernel =  strread((char(contents)),'%s','delimiter','\t');
clear contents cline

H  = str2num(parameters_kernel{1})*power(ma,str2num(parameters_kernel{2}));
h  = str2num(parameters_kernel{1})*power(length(dn),str2num(parameters_kernel{2}));

npairs  = 10000; % number of variable pairs to sample for unconditional MI
nbins   = 100;   % number of bins
nsub    = 1000;  % number of random subsampling for each sampled pair in each bin

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Sample un-conditional mutual information
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
mi = zeros(npairs,3);
it = 1;
disp(['Ramdom sampling ' num2str(npairs) ' pairs of genes and computing MI ... ']);
while (it <= npairs)
    if (mod(it*100/npairs,10) == 0), fprintf(1,'%d%s\n', it*100/npairs, '% completed.'), end
    ids = randsample(ng,2);
    if strcmp(method_mutualinfo,'fixed_bandwidth')
        foo = mutualinfo(data(ids(1),:),data(ids(2),:),H);
    else
        foo = mutualinfo_ap([data(ids(1),:);data(ids(2),:)]');
    end
    mi(it,:) = [ids(1) ids(2) foo];
    it = it+1;
end
clear global;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Generate null distribution of conditional mutual information
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
disp('Sorting the unconditional MI ... ');
% sort by unconditional mutual information
[foo,I]  = sort(mi(:,3));
bin_size = npairs/nbins;

null_dist    = zeros(bin_size*nsub,nbins,'single');
bin_stat = zeros(nbins,3);
for i=1:nbins
    disp(['Processing bin #' num2str(i) ' ... ']);
    bin_lb        = mi(I(bin_size*(i-1)+1),3);
    bin_ub        = mi(I(bin_size*i),3);
    bin_cn        = mi(I(bin_size*(i-0.5)),3);
    bin_stat(i,:) = [bin_lb bin_ub bin_cn];

    cur_bin = mi(I((bin_size*(i-1)+1):(bin_size*i)),:);
    for j=1:bin_size
        for k=1:nsub
            ptr = (j-1)*nsub+k;
            idx = randperm(ma);
            if strcmp(method_mutualinfo,'fixed_bandwidth')
                lo  = mutualinfo(data(cur_bin(j,1),idx(dn)),data(cur_bin(j,2),idx(dn)),h);
                hi  = mutualinfo(data(cur_bin(j,1),idx(up)),data(cur_bin(j,2),idx(up)),h);
            else
                lo  = mutualinfo_ap([data(cur_bin(j,1),idx(dn));data(cur_bin(j,2),idx(dn))]');
                hi  = mutualinfo_ap([data(cur_bin(j,1),idx(up));data(cur_bin(j,2),idx(up))]');
            end
            null_dist(ptr,i) = hi-lo;
        end
    end
end
clear global;

% save all the workspace variables
save('delta_mi_null_dist.mat');

