#!/usr/bin/perl
#
# written by Kai Wang (kw2110@columbia.edu)
#
use warnings;
use strict;

if($#ARGV != 1) {
    print "Usage: perl TestMINDY.pl <hub affyid> <condtion file>\n";
    exit(1);
}

my $hub = $ARGV[0];
my $infile = $ARGV[1];
my $aracne_dir = '/users/kw2110/project/_ARACNE/';
my $mapfile = '/users/kw2110/project/data/Bcell_110104_small_f50_affy2id.dat';
my $outfile = 'mindy_by_aracne.cam';
my $percent = 0.33;
my $tffile = '/users/kw2110/project/data/Bcell_110104_small_f50_transfac_831.dat';

my @candidates = ();
open(IN, "< $infile") || die "Could not open input file \"$infile\"!";
while (<IN>) {
    chomp;
	s/\r$//;
	push @candidates, $_;
}
close(IN);

print "==> Compute unconditional MIs ... \n";
system $aracne_dir."aracne -i Bcell_110104_small_f50.exp -h $hub -o uncondition_mi.adj -H $aracne_dir";
print "==> Done!\n\n";

foreach (@candidates) {
	print "==> Conditioning on $_ ...\n";
	system $aracne_dir."aracne -i Bcell_110104_small_f50.exp -h $hub -p 1e-7 -c +$_ $percent -e 0.1 -l $tffile -H $aracne_dir -o $_"."_up_cam.adj";
	system $aracne_dir."aracne -i Bcell_110104_small_f50.exp -h $hub -p 1e-7 -c -$_ $percent -e 0.1 -l $tffile -H $aracne_dir -o $_"."_dn_cam.adj";
	print "==> Done!\n\n";
}

open(MAP, "< $mapfile") || die "Could not open input file \"$mapfile\"!";
my $hash = <MAP>;
my %affy2id = split '\t', $hash;
close(MAP);

my @cmatrix;
for my $i (0 .. (7907-1) ) {
	for my $j (0 .. (scalar(@candidates)*2) ) {
    	$cmatrix[$i][$j] = 0;
    }
}

my $col = 0;
my $uncondfile = 'uncondition_mi.adj';
print "==> Processing $uncondfile ...\n";
open(MI, "< $uncondfile") || die "Could not open input file \"$uncondfile\"!";
while (<MI>) {
    if (/^>/) {
	    next;
	}
	chomp;
	s/\r$//;
	my @line = split '\t';
	if ($line[0] ne "1973_s_at") {
	    print "Go to hell!\n";
	}
	for (my $k=1; $k < scalar(@line); $k+=2) {
	    $cmatrix[$affy2id{$line[$k]}][$col] = $line[$k+1];
	}
}
close(MI);

foreach my $m (@candidates) {
	foreach my $c (("dn",'up')) {
	    my $file = $m."_".$c."_cam.adj";
		print "==> Processing $file ...\n";
		open(ADJ, "< $file") || die "Could not open input file \"$file\"!";
		$col++;
		while (<ADJ>) {
		    if (/^>/) {
			    next;
			}
			chomp;
			s/\r$//;
			my @line = split '\t';
			if ($line[0] ne "1973_s_at") {
			    print "Go to hell!\n";
			}
			for (my $p=1; $p < scalar(@line); $p+=2) {
			    $cmatrix[$affy2id{$line[$p]}][$col] = $line[$p+1];
			}
		}
		close(ADJ);
	}
	print "==> Done!\n\n";
}

print "\nOutputting to file ... \n";
open(OUT, "> $outfile");

# header line
print OUT "HUB::1973_s_at\tMI";
foreach (@candidates) {
	print OUT "\t$_:L\t$_:H";
}
print OUT "\n";

foreach (sort {$affy2id{$a} <=> $affy2id{$b}} keys %affy2id) {
	print OUT "$_";
	for my $itr (0 .. (scalar(@candidates)*2)) {
	    print OUT "\t$cmatrix[$affy2id{$_}][$itr]";
	}
	print OUT "\n";
}
close(OUT);
