/*
 * Copyright (C) 2003, 2004  Columbia Genome Center
 * All Rights Reserved.
 *
 * param.h --
 *
 * $Id: param.h,v 1.1 2009/07/15 01:50:10 manju Exp $
 */
#ifndef PARAM_H__
#define PARAM_H__

#include <string>
#include <vector>
#include "Microarray_Set.h"

using namespace std;

// Parameters for the algorithm
struct Parameter{

	double threshold;   // conditional mi threshold
    double Threshold;   // uncondtional mi threshold
    double pvalue;      // pvalue cutoff for conditional mi
    double Pvalue;      // pvalue cutoff for unconditional mi
    double sigma;       // gaussian kernel width for conditional mi
    double Sigma;       // gaussian kernel width for unconditional mi
    double eps;         // DPI tolerance
    double percent;     // high/low percentage for the conditional analysis
	double mean;        // filter mean
	double cv;          // filter stdev
    int nmarker;
    int nsample;
    int nsubsample;
    int ncandidate;
    int hubid;
    int miSteps;
    bool verbose;
	string algorithm;
    string infile;
    string outfile;
    string hub;
    string candidate_file;
    string tf_list_file;
    string home_dir;
    vector < int > tflist;

	Parameter()
	{
	    miSteps = 6;
    }
};

// service functions
string getFileExt(string matrixName);
string getFileName(string matrixName);
int cmpIgnoreCase(const char* a, const char* b);
void processParameter(Parameter &p, Microarray_Set & data, Microarray_Set & candidate);
void displayParameter(Parameter &p);
void readProbeList(string file, vector<int> & v, Microarray_Set & data);
double findKernelWidth(int n, string base);
double findThreshold(int n, double pvalue, string base);

#endif
