#!/usr/bin/perl
# This script will run the MINDY program on ADCLUSTER
#
# Usage: perl qsub_mindy <file containing list of TFs> <other MINDY paramters>
#
# Example: perl MINDY hub_list.dat -i microarray.exp -c conditions.dat -p 0.35 -k 0.15 -e 0.10 -t 0.08 -l subnet_nodes.dat
#
# Author: Kai Wang

use warnings;
use strict;

if ($#ARGV == -1) {
	print "Usage: perl qsub_mindy.pl <output dir> <file containing list of TFs> <other MINDY paramters>\n";
	print "Example: perl /users/kw2110/kinome qsub_mindy.pl transfactor_hub.lst -i data.exp -c candidate_modulator.lst -q 0.35 -p 1e-7 -e 0.10 -l transcription_factor.lst\n";
	exit(0);
}

my ($dir,$tflist,@otherarg)=@ARGV;
my $otherarg="@otherarg";
$dir =~ s/\/$//;
if (!(-d "$dir/junk")) {
	system "mkdir $dir/junk";
}

if (!(-d "$dir/soe")) {
	system "mkdir $dir/soe";
}

open(TF, "< $tflist") || die "Could not open input file \"$tflist\"!";

my $cnt = 1;
while (<TF>) {
	chomp;
	s/\r$//;
	my ($hubid) = split '\t';
	my $scrfile = "$dir/junk/mindy_run_".$cnt.".scr";
	open(SCR, "> $scrfile");
	print SCR "#! /bin/sh\n";
	print SCR "#\$ \-S /bin/sh -cwd\n";
	print SCR "#\$ \-o $dir/soe/mindy_run_".$cnt.".soe -j y\n";
	print SCR "./mindy -h $hubid $otherarg";
	close(SCR);
	$cnt++;
	system "chmod +x $scrfile";
	system "qsub $scrfile";
	sleep 1;
}
close(TF);

my ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) = localtime(time);
$year += 1900;
$mon += 1;
my $foo = $hour%12;
my $bar = 'am';
if (int($hour/12) == 1) {
    $bar = 'pm';
}

my $logfile = "command_".$mon."_".$mday."_".$year."_".$foo.$bar.".log";
open(LOG, "> $logfile");
print LOG "HUB list: $tflist\n";
print LOG "MINDY parameters: $otherarg\n";
close(LOG);
